﻿// Copyright © Serilog Contributors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// ReSharper disable ForCanBeConvertedToForeach

namespace Serilog.Expressions.Compilation.Linq;

/// <summary>
/// Hash functions for message templates. See <see cref="Compute"/>.
/// </summary>
public static class EventIdHash
{
    /// <summary>
    /// Compute a 32-bit hash of the provided <paramref name="messageTemplate"/>. The
    /// resulting hash value can be uses as an event id in lieu of transmitting the
    /// full template string.
    /// </summary>
    /// <param name="messageTemplate">A message template.</param>
    /// <returns>A 32-bit hash of the template.</returns>
    [CLSCompliant(false)]
    public static uint Compute(string messageTemplate)
    {
        if (messageTemplate == null) throw new ArgumentNullException(nameof(messageTemplate));

        // Jenkins one-at-a-time https://en.wikipedia.org/wiki/Jenkins_hash_function
        unchecked
        {
            uint hash = 0;
            for (var i = 0; i < messageTemplate.Length; ++i)
            {
                hash += messageTemplate[i];
                hash += hash << 10;
                hash ^= hash >> 6;
            }
            hash += hash << 3;
            hash ^= hash >> 11;
            hash += hash << 15;
            return hash;
        }
    }
}